"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var SpreadsheetCell = /** @class */ (function () {
    function SpreadsheetCell(spreadsheet, worksheet_id, data) {
        var _this = this;
        // --------- PUBLIC API ---------------
        /**
         * Sets value and saves
         * @param {string} new_value
         * @returns {Promise<void>}
         */
        this.setValue = function (new_value) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.val(new_value);
                return [2 /*return*/, this.save()]; //todo no callback
            });
        }); };
        /**
         * Saves cell. Mutates cell if e.g. a formula has been entered-
         * @returns {Promise<void>}
         */
        this.save = function () { return __awaiter(_this, void 0, void 0, function () {
            var data_xml, response;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        data_xml = '<entry><id>' +
                            this.id +
                            '</id>' +
                            '<link rel="edit" type="application/atom+xml" href="' +
                            this.id +
                            '"/>' +
                            '<gs:cell row="' +
                            this.row +
                            '" col="' +
                            this.col +
                            '" inputValue="' +
                            this.valueForSave() +
                            '"/></entry>';
                        data_xml = data_xml.replace('<entry>', "<entry xmlns='http://www.w3.org/2005/Atom' xmlns:gs='http://schemas.google.com/spreadsheets/2006'>");
                        return [4 /*yield*/, this.spreadsheet.makeFeedRequest(this['_links']['edit'], 'PUT', data_xml)];
                    case 1:
                        response = _a.sent();
                        this.updateValuesFromResponseData(response.data);
                        return [2 /*return*/];
                }
            });
        }); };
        /**
         * Clears cell
         * @returns {Promise<Promise<void>>}
         */
        this.del = function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.setValue('')];
            });
        }); };
        // TODO : figure out how to enforce that you can set with both string and number, but you always get a string back.
        // Value overload taking both string and number
        this.val = function (val) {
            //Enforce stringiness
            _this._value = val.toString();
            if (typeof val == 'string' && val.substr(0, 1) === '=') {
                // use the getter to clear the value
                _this.formula = val;
            }
            else {
                _this._formula = undefined;
            }
        };
        // ------------ Private ---------------
        this.updateValuesFromResponseData = function (_data) {
            // formula value
            var input_val = _data['gs:cell']['$']['inputValue'];
            // inputValue can be undefined so substr throws an error
            // still unsure how this situation happens
            if (input_val && input_val.substr(0, 1) === '=') {
                _this._formula = input_val;
            }
            else {
                _this._formula = undefined;
            }
            // the main "value" - its always a string
            _this._value = _data['gs:cell']['_'] || '';
        };
        this._clearValue = function () {
            _this._formula = undefined;
            _this._value = '';
        };
        this.spreadsheet = spreadsheet;
        this.worksheet_id = worksheet_id;
        this.id = data.id;
        this.row = parseInt(data['gs:cell'].$.row);
        this.col = parseInt(data['gs:cell'].$.col);
        this.batchId = 'R' + this.row + 'C' + this.col;
        this._links = [];
        var links = utils_1.forceArray(data.link);
        links.forEach(function (link) {
            _this._links[link.$.rel] = link.$.href;
        });
        this.updateValuesFromResponseData(data);
    }
    Object.defineProperty(SpreadsheetCell.prototype, "value", {
        // GETTERS AND SETTERS
        get: function () {
            return this._value;
        },
        /**
         * Set the value. If you want to clear, either use the empty string or use the .del method
         * @param {string | number} val
         */
        set: function (val) {
            if (!val) {
                this._clearValue();
                return;
            }
            this.val(val);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SpreadsheetCell.prototype, "formula", {
        get: function () {
            return this._formula;
        },
        // Set the formula. Use del or empty string to clear
        set: function (val) {
            if (!val) {
                this._clearValue();
                return;
            }
            if (val.substr(0, 1) !== '=') {
                throw new Error('Formulas must start with "="');
            }
            this._value = '*SAVE TO GET NEW VALUE*';
            this._formula = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SpreadsheetCell.prototype, "numericValue", {
        get: function () {
            var valAsNumber = parseFloat(this._value);
            if (!valAsNumber) {
                throw new Error("Attempted to get numeric value of a non-numeric field. Field value was: '" + this._value + "'");
            }
            return valAsNumber;
        },
        enumerable: true,
        configurable: true
    });
    SpreadsheetCell.prototype.valueForSave = function () {
        return utils_1.xmlSafeValue(this._formula || this._value);
    };
    return SpreadsheetCell;
}());
exports.SpreadsheetCell = SpreadsheetCell;
//# sourceMappingURL=SpreadsheetCell.js.map